#include <avr\io.h>
#include <avr\interrupt.h>

#define ENGINE1_DDR  DDRA
#define ENGINE1_PORT PORTA
#define EN1 PA0
#define INPUT1 PA1
#define PWM1_DDR  DDRD
#define PWM1_PORT PORTD
#define PWM1 PD5
#define ENGINE2_DDR  DDRA
#define ENGINE2_PORT PORTA
#define EN2 PA3
#define INPUT2 PA4
#define PWM2_DDR  DDRD
#define PWM2_PORT PORTD
#define PWM2 PD4
#define SENSOR_DDR DDRC
#define SENSOR_PIN PINC
#define S1 PC0
#define S2 PC1
#define S3 PC2

#define OUT_OF_LINE (sensors == 0)
#define OUT_CENTER_RIGHT (sensors == 1)
#define OUT_CENTER_LEFT (sensors == 4)
#define OUT_RIGHT (sensors == 3)
#define OUT_LEFT (sensors == 6)
#define ON_LINE (sensors == 7)

void stop1()
{
	OCR1A = 0;
	ENGINE1_PORT &= ~(_BV(INPUT1));
}

void stop2()
{
	OCR1B = 0;
	ENGINE2_PORT &= ~(_BV(INPUT2));
}


void moveFront1(float speed)
{
	ENGINE1_PORT &= ~(_BV(INPUT1));
	OCR1A = (int)(255 * speed);
}

void moveFront2(float speed)
{
	ENGINE2_PORT &= ~(_BV(INPUT2));
	OCR1B = (int)(255 * speed);
}

void moveBack1()
{
	ENGINE1_PORT |= _BV(INPUT1);
	OCR1A = 0;
}

void moveBack2()
{
	ENGINE2_PORT |= _BV(INPUT2);
	OCR1B = 0;
}

void turnLeftHard()
{
	stop1();
	moveFront2(1);
}

void turnRightHard()
{
	stop2();
	moveFront1(1);
}

void turnLeftSoft()
{
	moveFront1(0.5);
	moveFront2(1);
}

void turnRightSoft()
{
	moveFront2(0.5);
	moveFront1(1);
}

void init()
{
	TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(WGM10);
	TCCR1B = _BV(WGM12) | _BV(CS10) | _BV(CS12);
	ENGINE1_DDR = ENGINE2_DDR = PWM1_DDR = PWM2_DDR = SENSOR_DDR = 0;
	ENGINE1_DDR |= _BV(EN1) | _BV(INPUT1);
	ENGINE2_DDR |= _BV(EN2) | _BV(INPUT2);
	PWM1_DDR |= _BV(PWM1);
	PWM2_DDR |= _BV(PWM2);
	SENSOR_PIN &= ~(_BV(S1) | _BV(S2) | _BV(S3));
	ENGINE1_PORT |= _BV(EN1);
	ENGINE2_PORT |= _BV(EN2);
	moveFront1(1);
	moveFront2(1);
}
	


int main()
{
	init();
	unsigned char sensors;
	while (1)
	{
		sensors = SENSOR_PIN & (_BV(S1) | _BV(S2) | _BV(S3));
		if (OUT_LEFT)
		{
			turnRightSoft();
			continue;
		}
		if (OUT_RIGHT)
		{
			turnLeftSoft();
			continue;
		}
		if (OUT_CENTER_LEFT)
		{
			turnRightHard();
			continue;
		}
		if (OUT_CENTER_RIGHT)
		{
			turnLeftHard();
			continue;
		}
		if (OUT_OF_LINE)
		{
			moveBack1();
			moveBack2();
			continue;
		}
		moveFront1(1);
		moveFront2(1);
	}

	return 0;
}



