#include <avr\io.h>
#include <avr\interrupt.h>

#define ENGINE1_DDR  DDRA
#define ENGINE1_PORT PORTA
#define EN1 PA0
#define INPUT1 PA1
#define INPUT2 PA2
#define ENGINE2_DDR  DDRA
#define ENGINE2_PORT PORTA
#define EN2 PA3
#define INPUT3 PA4
#define INPUT4 PA5
#define SENSOR_DDR DDRC
#define SENSOR_PIN PINC
#define S1 PC0
#define S2 PC1
#define S3 PC2

#define OUT_OF_LINE (sensors == 0)
#define OUT_CENTER_RIGHT (sensors == 1)
#define OUT_CENTER_LEFT (sensors == 4)
#define OUT_RIGHT (sensors == 3)
#define OUT_LEFT (sensors == 6)
#define ON_LINE (sensors == 7)

void stop1()
{
	ENGINE1_PORT &= ~(_BV(EN1) | _BV(INPUT1) | _BV(INPUT2));
	ENGINE1_PORT |= _BV(EN1);
}

void stop2()
{
	ENGINE2_PORT &= ~(_BV(EN2) | _BV(INPUT3) | _BV(INPUT4));
	ENGINE2_PORT |= _BV(EN2);
}


void moveFront1(float speed)
{
	ENGINE1_PORT &= ~(_BV(EN1) | _BV(INPUT1) | _BV(INPUT2));
	ENGINE1_PORT |= _BV(EN1) | _BV(INPUT1);
}

void moveFront2(float speed)
{
	ENGINE2_PORT &= ~(_BV(EN2) | _BV(INPUT3) | _BV(INPUT4));
	ENGINE2_PORT |= _BV(EN2) | _BV(INPUT4);
}

void moveBack1()
{
	ENGINE1_PORT &= ~(_BV(EN1) | _BV(INPUT1) | _BV(INPUT2));
	ENGINE1_PORT |= _BV(EN1) | _BV(INPUT2);
}

void moveBack2()
{
	ENGINE2_PORT &= ~(_BV(EN2) | _BV(INPUT3) | _BV(INPUT4));
	ENGINE2_PORT |= _BV(EN2) | _BV(INPUT3);
}

void turnLeftHard()
{
	stop1();
	moveFront2(1);
}

void turnRightHard()
{
	stop2();
	moveFront1(1);
}

void turnLeftSoft()
{
	moveFront1(0.5);
	moveFront2(1);
}

void turnRightSoft()
{
	moveFront2(0.5);
	moveFront1(1);
}

void init()
{
	ENGINE1_DDR = ENGINE2_DDR = SENSOR_DDR = 0;
	ENGINE1_PORT |= _BV(EN1) | _BV(INPUT1) | _BV(INPUT2);
	ENGINE2_PORT |= _BV(EN2) | _BV(INPUT3) | _BV(INPUT4);
	SENSOR_PIN &= ~(_BV(S1) | _BV(S2) | _BV(S3));
	moveFront1(1);
	moveFront2(1);
}
	


int main()
{
	init();
	unsigned char sensors;
	while (1)
	{
		sensors = SENSOR_PIN & (_BV(S1) | _BV(S2) | _BV(S3));
		if (OUT_LEFT)
		{
			turnRightSoft();
			continue;
		}
		if (OUT_RIGHT)
		{
			turnLeftSoft();
			continue;
		}
		if (OUT_CENTER_LEFT)
		{
			turnRightHard();
			continue;
		}
		if (OUT_CENTER_RIGHT)
		{
			turnLeftHard();
			continue;
		}
		if (OUT_OF_LINE)
		{
			moveBack1();
			moveBack2();
			continue;
		}
		moveFront1(1);
		moveFront2(1);
	}

	return 0;
}



